#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'program_information/toplevel_methods/return_proper_pipe_token_to_this_input.rb'
# =========================================================================== #
require 'program_information/constants/regex.rb'
require 'program_information/toplevel_methods/opnn.rb'
require 'program_information/toplevel_methods/remove_archive_type.rb'

module ProgramInformation

  # ========================================================================= #
  # === ProgramInformation.return_proper_pipe_token_to_this_input
  #
  # Most of the content of this method has to be put into a begin/require
  # clause, as errors may happen, which could propagate downstream,
  # leading to confusion. So we will rescue the error, and notify the
  # user as to what has happened.
  # ========================================================================= #
  def self.return_proper_pipe_token_to_this_input(
      entry
    )
    if entry.is_a? Array
      entry = entry.first
    end
    entry = remove_archive_type(entry)
    use_this_replacement_token = '|'
    return if entry.to_s.empty?
    begin
      # ===================================================================== #
      # === Check for input such as 'unieject-6', containing one '-' and 0 '_'
      # ===================================================================== #
      if (entry.count('-') == 1) and
         (entry.count('_') == 0)
        entry[entry.index('-'),1] = use_this_replacement_token
      # ===================================================================== #
      # === Check for input such as 'p7zip_16.02_src_all.tar.bz2' with
      #     3 '_' and at the least one '.' and no '-'
      # ===================================================================== #
      elsif (entry.count('_') == 3) and
            (entry.count('.') > 0) and
            (!entry.include?('-'))
        entry[entry.index('_'),1] = use_this_replacement_token
      # ===================================================================== #
      # === Input such as 'mod_python_3.3.1', with two '_' and no '-'.
      # ===================================================================== #
      elsif (entry.count('_') == 2) and
            (entry.count('-') == 0)
        entry[entry.rindex('_'),1] = use_this_replacement_token
      # ===================================================================== #
      # === Input such as 'unzip60' without any '-' or '_'
      # ===================================================================== #
      elsif (entry.count('-') == 0) and
            (entry.count('_') == 0) and
            (!entry.include?('.'))
        entry[entry.index(/\d/), 0] = '|'
      # ===================================================================== #
      # === Check for input such as 'artemis_v16.0.11' or 'dpkg_1.18.22'
      #     with one '_' and no '-'.
      # ===================================================================== #
      elsif (entry.count('_') == 1) and
            (entry.count('-') == 0)
        entry[entry.index('_'),1] = use_this_replacement_token
      # ===================================================================== #
      # === Input such as 'sendmail.8.15.2' or 'expect5.45.3', without
      #     any '-' or '_'.
      # ===================================================================== #
      elsif (entry.count('-') == 0) and
            (entry.count('_') == 0) and
            (entry.include?('.'))
        entry =~ REGEX_FOR_INPUT_HAVING_ONLY_DOTS
        entry = $1.to_s.dup+use_this_replacement_token+$2.to_s.dup
      # ===================================================================== #
      # For input such as 'boost-1_54_0' or 'ncbi_cxx-12_0_0' or
      # 'boost_1.56.6'. So it contains exactly one '-' character.
      # ===================================================================== #
      elsif ((entry.count('-') == 1) and
            (entry.count('_') > 1) and # More than 1 '_'
            (entry =~ REGEX_FOR_BOOST_LIKE_INPUT)) or # or the second condition.
            ((entry.count('_') == 3) and
            (entry.count('-') == 0) and
            (entry =~ REGEX_FOR_BOOST_LIKE_INPUT))
        entry =~ REGEX_FOR_BOOST_LIKE_INPUT
        # =================================================================== #
        # We here need only the first and the third match.
        # =================================================================== #
        entry = "#{$1.to_s.dup}#{use_this_replacement_token}#{$3.to_s.dup}"
      # ===================================================================== #
      # For input such as 'acl_2.2.47-1' or 'e_dbus-1.7.10' or
      # 'SDL2_gfx-1.0.1' or 'domain_name-0.5.20170404.gem' or
      # 'libatomic_ops-7.4.2' or 'icu4c-62_1'.
      # ===================================================================== #
      elsif (entry.count('_') == 1) and
            (entry.count('-') == 1) and
            (entry =~ REGEX_FOR_ACL_LIKE_INPUT)
        regex_to_use = REGEX_FOR_ACL_LIKE_INPUT
        entry =~ regex_to_use
        # =================================================================== #
        # Here we also only need to match towards the first and the third
        # entry.
        # =================================================================== #
        entry = $1.to_s.dup+use_this_replacement_token+$3.to_s.dup
      # ===================================================================== #
      # === Input such as 'did_you_mean-1.1.2' with one '-' and two '_'
      # ===================================================================== #
      elsif (entry.count('-') == 1) and
            (entry.count('_') == 2)
        entry[entry.index('-'),1] = use_this_replacement_token
      # ===================================================================== #
      # === Input such as 'ntfs-3g_ntfsprogs-2017.3.23' for 2 '-' and
      #     1 '_'.
      # ===================================================================== #
      elsif (entry.count('-') == 2) and
            (entry.count('_') == 1)
        regex_to_use = REGEX_FOR_INPUT_CONTAINING_TWO_HYPHENS_AND_ONE_UNDERSCORE
        entry =~ regex_to_use
        entry = $1.to_s.dup+use_this_replacement_token+$3.to_s.dup
      # ===================================================================== #
      # For input such as 'lynx-2.8.8-rel2' or 'subtle-0.11.3224-xi' or
      # 'gobject-introspection-1.53.4' or 'dvd+rw-tools-7.1' or
      # 'Archive-Zip-1.46' or 'b43-fwcutter-015' or 'wcd-6.0.1-beta3'
      # or 'oxygen-icons5-5.37.0'. These entries have two '-' and zero
      # '_'.
      # ===================================================================== #
      elsif (entry.count('-') == 2) and
            (entry.count('_') == 0)
        regex_to_use = REGEX_FOR_TWO_HYPHENS
        entry =~ regex_to_use
        entry = $1.to_s.dup+use_this_replacement_token+$2.to_s.dup
      # ===================================================================== #
      # For input such as 'polkit-qt-1-0.112.0' or
      # 'font-bitstream-75dpi-1.0.3' or 'boost-build-2014-10' or
      # 'xf86-video-i810-1.7.4' or 'gnome-internet-radio-locator-0.9.0'
      # or 'mime-types-data-3.2016.0521' or 'polkit-kde-agent-1-5.15.2'
      # or 'xf86-video-amd-2.7.7.7'.
      #
      # This input will NOT have any '_' token; and at the least 3
      # '-' tokens.
      # ===================================================================== #
      elsif (!entry.include?('_')) and
            (entry.count('-') >= 3)
        regex_to_use = REGEX_FOR_THREE_OR_MORE_HYPHENS
        entry =~ regex_to_use
        entry = $1.to_s.dup+
                use_this_replacement_token+
                $2.to_s.dup
      else
        e 'ERROR in program_information.rb! This should never happen.'
        e "The input was: #{entry}"
      end
    rescue Exception => error
      opnn; e 'An error happened (line: '+__LINE__.to_s+'), for '\
              'the entry `'+entry.to_s+'`.'
      pp error
      pp error.class
      opnn; e 'The caller stack was:'
      pp caller()
    end
    return entry
  end

end

if __FILE__ == $PROGRAM_NAME
  ProgramInformation.return_proper_pipe_token_to_this_input(ARGV)
end # rb return_proper*rb icu4c-61_1