#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'program_information/www/embeddable_interface.rb'
# include ProgramInformation::EmbeddableInterface
# =========================================================================== #
module ProgramInformation

module EmbeddableInterface

  # ========================================================================= #
  # === ProgramInformation::EmbeddableInterface.routes?
  #
  # Define all legal routes via this Array. This Array will then be used
  # to add more routes to any sinatra-application that needs it.
  # ========================================================================= #
  def self.routes?
    [
      'program_information',
      'program_information/*'
    ]
  end

  # ========================================================================= #
  # === return_sinatra_program_information
  # ========================================================================= #
  def return_sinatra_program_information
    'Please provide the name of the target program that is to be split.'
  end

  # ========================================================================= #
  # === return_sinatra_repackage_with_arguments
  # ========================================================================= #
  def return_sinatra_program_information_with_arguments(
      i = web_params_as_string?
    )
    i = i.dup if i.frozen?
    _ = ''.dup
    _ << 'The program information for: <b style="darkblue">'+i+'</b> is:<br>'
    i.prepend('/') unless i.start_with? '/'
    program_information = ::ProgramInformation.new(i)
    program_name = program_information.name?
    program_version = program_information.version?
    _ << 'Program name: <b>'+program_name.to_s+'</b><br>'
    _ << 'Program version: <b>'+program_version.to_s+'</b><br>'
    return _
  end

end

# =========================================================================== #
# === ProgramInformation.embeddable_interface
# =========================================================================== #
def self.embeddable_interface
  object = Object.new
  object.extend(::ProgramInformation::EmbeddableInterface)
  return object
end

end